/*
** Barone-Adesi, G. and R. E. Whaley [1987], Efficient analytic
** approximation of american option values, Journal of Finance,
** 42, pages 301-320
*/

new;
library option;

K = 100;
e = ones(5,1);
r = (0.08|0.12|0.08|0.08).*.e;
sigma = (0.20|0.20|0.40|0.20).*.e;
tau = (0.25|0.25|0.25|0.50).*.e;
S0 = ones(4,1).*.seqa(80,10,5);

begin_ct = hsec;

/* Table I */

b = -0.04*ones(20,1);

_Option_Type = "call";
C = AmericanBS(S0,K,sigma,tau,b,r);
_Option_Type = "put";
P = AmericanBS(S0,K,sigma,tau,b,r);

Print "                          Table I       ";
call Print_Table(b,r,sigma,tau,S0,C,P);


/* Table II */

b = 0.04*ones(20,1);

_Option_Type = "call";
C = AmericanBS(S0,K,sigma,tau,b,r);
_Option_Type = "put";
P = AmericanBS(S0,K,sigma,tau,b,r);

Print "                          Table II       ";
call Print_Table(b,r,sigma,tau,S0,C,P);


/* Table III */

b = 0.00*ones(20,1);

_Option_Type = "call";
C = AmericanBS(S0,K,sigma,tau,b,r);
_Option_Type = "put";
P = AmericanBS(S0,K,sigma,tau,b,r);

Print "                          Table III       ";
call Print_Table(b,r,sigma,tau,S0,C,P);


/* Table IV */

b = r;
_Option_Type = "call";
C = AmericanBS(S0,K,sigma,tau,b,r);
_Option_Type = "put";
P = AmericanBS(S0,K,sigma,tau,b,r);

Print "                          Table IV       ";
call Print_Table(b,r,sigma,tau,S0,C,P);

/* Table V */

b = (-0.04|0.00|0.04|0.08).*.e;
r = (0.08|0.08|0.08|0.08).*.e;
sigma = (0.20|0.20|0.20|0.20).*.e;
tau = (3|3|3|3).*.e;

_Option_Type = "call";
C = AmericanBS(S0,K,sigma,tau,b,r);
_Option_Type = "put";
P = AmericanBS(S0,K,sigma,tau,b,r);

Print "                          Table V       ";
call Print_Table(b,r,sigma,tau,S0,C,P);

end_ct = hsec;

print ftos((end_ct-begin_ct)/100,"Computational Time: %lf seconds",5,3);


proc (0) = Print_Table(b,r,sigma,tau,S0,C,P);
  local mask,fmt,omat;

  mask = 1;
  let fmt[7,3]=   "*.*lf" 8 2 "*.*lf" 8 2  "*.*lf" 8 2
                  "*.*lf" 8 2 "*.*lf" 8 0 "*.*lf" 15 3  "*.*lf" 15 3;

  omat = b~r~sigma~tau~S0~C~P;
  print "     b       r     sigma    tau       S0          Call           Put";
  print "------------------------------------------------------------------"\
        "-----";
  call printfm(omat,mask,fmt);
  print;  print;
  retp;
endp;




