new;
library option,pgraph;

dlibrary option_tridiag; // Don't miss to download the tridiag solver

cls;

K = 100;
sigma = 0.20;
tau = 0.25;
r = 0.08;
b = -0.04;

proc aProc(t,x);
  retp( 0.5*(sigma^2) .* (x^2) );
endp;

proc bProc(t,x);
  retp( b .* x );
endp;

proc cProc(t,x);
  retp( r .* ones(rows(x),1) );
endp;

proc dProc(t,x);
  retp( zeros(rows(x),1) );
endp;

proc tminBound(t,x);
  local PayOff;
  PayOff = x - K;
  PayOff = PayOff .* (PayOff .> 0);
  retp( PayOff );
endp;

proc xminBound(t,x);
  retp( zeros(rows(x),1) );
endp;

proc xmaxBound(t,x);
  local PayOff;
  PayOff = x - K;
  PayOff = PayOff .* (PayOff .> 0);
  retp( PayOff );
endp;

proc DxmaxBound(t,x);
  retp( ones(rows(x),1) );
endp;


xmin = 50; xmax = 150; Nx = 201;
tmin = 0; tmax = tau; Nt = 1000;

indx = 91|101|111;

_PDE_SaveLastIter = 1;
_PDE_PrintIters = 50;

U = zeros(Nx,3);

/* Dirichlet */

call PDE(&aProc,&bProc,&cProc,&dProc,0,
         &tminBound,&xminBound,&xmaxBound,0,0);
call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,"pde",0.5);
tau = PDE_readFile("pde","t");
x = PDE_readFile("pde","x");
U[.,1] = PDE_readFile("pde","t"|tau);

/* Neumann */

call PDE(&aProc,&bProc,&cProc,&dProc,0,
         &tminBound,&xminBound,0,0,0);
call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,"pde",0.5);
U[.,2] = PDE_readFile("pde","t"|tau);

/* Neumann "user" */

call PDE(&aProc,&bProc,&cProc,&dProc,0,
         &tminBound,&xminBound,0,0,&DxmaxBound);
call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,"pde",0.5);
U[.,3] = PDE_readFile("pde","t"|tau);

C = EuropeanBS(x,K,sigma,tau,b,r);

call DeleteFile("pde.dat");

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;

  begwind;
  makewind(9,6.855,0,0,1);
  makewind(4,4,1.5,2,1);
 
  setwind(1);
    xtics(50,160,10,0);
    ytics(0,60,10,10);
    _pltype = 6|3|1|5;
    _pline = 1~6~140~35~151~35~0~7~10 |
             1~6~140~51~151~51~0~7~10 |
             1~6~140~35~140~51~0~7~10 |
             1~6~151~35~151~51~0~7~10 ;
    _plegstr = "Dirichlet\000Neumann\000"\
               "Neumann 'user'\000BS formula";
    _plegctl = {2 5 5 1};
    ylabel("\216C");
    xlabel("\216S");
    xy(x,U~C);
  setwind(2);
    graphset;
    _pnumht = 0.25; _pnum = 2; _paxht = 0.25; _pltype = 6|3|1|5;
    xtics(140,150,1,0);
    ytics(-3,8,1,0);
    xlabel("\216S");
    ylabel("\216Numerical error");
    xy(x,C-U);
  endwind;

